package scm.sva.framework.vag.base;


import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager2;
import java.awt.Rectangle;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;

/**
 * @author tej
 */
public class STLayout implements LayoutManager2 {

	/**
	 * @author tej
	 * 
	 */
	private class STBounds {

		private int location;

		private int length;

		public STBounds(int location, int length) {
			super();
			this.location = location;
			this.length = length;
		}

		public int getLength() {
			return length;
		}

		public int getLocation() {
			return location;
		}
	}

	/**
	 * @author tej
	 * 
	 */
	private class STEdgeCache {

		private Map<Component, Double> bottom = new Hashtable<Component, Double>();

		private Map<Component, Double> left = new Hashtable<Component, Double>();

		private Map<Component, Double> right = new Hashtable<Component, Double>();

		private Map<Component, Double> top = new Hashtable<Component, Double>();

		public STEdgeCache() {
			super();
		}

		public void addBottom(Component c, double d) {
			bottom.put(c, d);
		}

		public void addLeft(Component c, double d) {
			left.put(c, d);
		}

		public void addRight(Component c, double d) {
			right.put(c, d);
		}

		public void addTop(Component c, double d) {
			top.put(c, d);
		}

		public boolean containsBottom(Component c) {
			return bottom.containsKey(c);
		}

		public boolean containsLeft(Component c) {
			return left.containsKey(c);
		}

		public boolean containsRight(Component c) {
			return right.containsKey(c);
		}

		public boolean containsTop(Component c) {
			return top.containsKey(c);
		}

		public double getBottom(Component c) {
			return bottom.get(c);
		}

		public double getLeft(Component c) {
			return left.get(c);
		}

		public double getRight(Component c) {
			return right.get(c);
		}

		public double getTop(Component c) {
			return top.get(c);
		}

	}

	private Map<Component, FramingSpec> components;

	private boolean update = false;

	/**
	 * 
	 */
	public STLayout() {
		super();
		this.components = new HashMap<Component, FramingSpec>();
	}

	/**
	 * @see java.awt.LayoutManager2#addLayoutComponent(java.awt.Component, java.lang.Object)
	 */
	public void addLayoutComponent(Component comp, Object constraints) {
		if (constraints == null || constraints instanceof FramingSpec) {
			this.components.put(comp, (FramingSpec) constraints);
			if (constraints != null) {
				FramingSpec f = (FramingSpec) constraints;
				if (f.getBottomTarget() == comp || f.getTopTarget() == comp || f.getLeftTarget() == comp || f.getRightTarget() == comp) {
					throw new IllegalArgumentException();
				}
			}
		} else {
			throw new IllegalArgumentException("constraints must be instance of " + FramingSpec.class.getName());
		}
	}

	/**
	 * @see java.awt.LayoutManager#addLayoutComponent(java.lang.String, java.awt.Component)
	 */
	public void addLayoutComponent(String name, Component comp) {
		throw new UnsupportedOperationException();
	}

	/**
	 * @param f
	 * @param parent
	 * @param top
	 * @return
	 */
	private double calculateBottom(Component c, FramingSpec f, Dimension parent, double top, STEdgeCache cache) {

		if (cache.containsBottom(c)) {
			return cache.getBottom(c);
		}

		if (!FramingSpec.isSupported(f.getBottomEdgeAttachment()) || f.getBottomEdgeAttachment() == FramingSpec.XmATTACHNONE) {
			return top + this.getPreferredHeight(c);
		}
		if (f.getBottomEdgeAttachment() == FramingSpec.XmATTACHFORM) {
			return parent.getHeight() - f.getBottomEdge();
		}
		if (f.getBottomEdgeAttachment() == FramingSpec.XmATTACHOPPOSITEFORM) {
			return f.getBottomEdge();
		}
		if (f.getBottomEdgeAttachment() == FramingSpec.XmATTACHSELFOPPOSITE) {
			return top + f.getBottomEdge();
		}
		if (f.getBottomEdgeAttachment() == FramingSpec.XmATTACHWIDGET) {
			if (f.getBottomTarget() == null) {
				return parent.getHeight() - f.getBottomEdge();
			} else {
				FramingSpec f2 = (FramingSpec) this.getFramingSpec(f.getBottomTarget());
				STBounds bounds = this.calculateBoundsTopBottom(f.getBottomTarget(), f2, this.getSizeOfParentOfAttachedWidget(f.getBottomTarget()), cache);
				return bounds.getLocation() - f.getBottomEdge();
			}
		}
		if (f.getBottomEdgeAttachment() == FramingSpec.XmATTACHOPPOSITEWIDGET) {
			if (f.getBottomTarget() == null) {
				return parent.getHeight() + f.getBottomEdge();
			} else {
				FramingSpec f2 = (FramingSpec) this.getFramingSpec(f.getBottomTarget());
				STBounds bounds = this.calculateBoundsTopBottom(f.getBottomTarget(), f2, this.getSizeOfParentOfAttachedWidget(f.getBottomTarget()), cache);
				return bounds.getLocation() + bounds.getLength() - f.getBottomEdge();
			}
		}
		if (f.getBottomEdgeAttachment() == FramingSpec.XmATTACHPOSITION) {
			return parent.getHeight() / 100.0 * f.getBottomEdge();
		}
		throw new RuntimeException("unsupported bottom attachment: " + f.getBottomEdgeAttachment());
	}

	/**
	 * @param f
	 * @param parent
	 * @return
	 */
	private Rectangle calculateBounds(Component c, FramingSpec f, Dimension parent, STEdgeCache cache) {
		STBounds horiz = this.calculateBoundsLeftRight(c, f, parent, cache);
		STBounds vert = this.calculateBoundsTopBottom(c, f, parent, cache);
		Rectangle bounds = new Rectangle(horiz.getLocation(), vert.getLocation(), horiz.getLength(), vert.getLength());
		return bounds;
	}

	/**
	 * @param c
	 * @param f
	 * @param parent
	 * @param cache
	 * @return
	 */
	private STBounds calculateBoundsLeftRight(Component c, FramingSpec f, Dimension parent, STEdgeCache cache) {
		double left = 0;
		double right = 0;
		if (f.getLeftEdgeAttachment() == FramingSpec.XmATTACHSELFOPPOSITE) {
			right = this.calculateRight(c, f, parent, left, cache);
			cache.addRight(c, right);
			left = this.calculateLeft(c, f, parent, right, cache);
			cache.addLeft(c, left);
		} else {
			left = this.calculateLeft(c, f, parent, right, cache);
			cache.addLeft(c, left);
			right = this.calculateRight(c, f, parent, left, cache);
			cache.addRight(c, right);
		}
		// berechne x und breite; ist die linke Kante nicht definiert (NULL oder
		// XmAttachNone), mssen x und breite vertauscht
		// werden
		int x = 0;
		int width = 0;
		if (!FramingSpec.isSupported(f.getLeftEdgeAttachment()) || f.getLeftEdgeAttachment() == FramingSpec.XmATTACHNONE) {
			x = (int) (right - left + 0.99999);
			width = (int) (left + 0.99999);
		} else {
			x = (int) (left + 0.99999);
			width = (int) (right - left + 0.99999);
		}

		return new STBounds(x, width);
	}

	/**
	 * @param c
	 * @param f
	 * @param parent
	 * @param cache
	 * @return
	 */
	private STBounds calculateBoundsTopBottom(Component c, FramingSpec f, Dimension parent, STEdgeCache cache) {
		double top = 0;
		double bottom = 0;
		if (f.getTopEdgeAttachment() == FramingSpec.XmATTACHSELFOPPOSITE) {
			bottom = this.calculateBottom(c, f, parent, top, cache);
			cache.addBottom(c, bottom);
			top = this.calculateTop(c, f, parent, bottom, cache);
			cache.addTop(c, top);
		} else {
			top = this.calculateTop(c, f, parent, bottom, cache);
			cache.addTop(c, top);
			bottom = this.calculateBottom(c, f, parent, top, cache);
			cache.addBottom(c, bottom);
		}
		// berechne y und hhe; ist die obere Kante nicht definiert (NULL oder
		// XmAttachNone), mssen y und hhe vertauscht
		// werden
		int y = 0;
		int height = 0;
		if (!FramingSpec.isSupported(f.getTopEdgeAttachment()) || f.getTopEdgeAttachment() == FramingSpec.XmATTACHNONE) {
			y = (int) (bottom - top + 0.99999);
			height = (int) (top + 0.99999);
		} else {
			y = (int) (top + 0.99999);
			height = (int) (bottom - top + 0.99999);
		}
		return new STBounds(y, height);
	}

	/**
	 * @param f
	 * @param parent
	 * @param right
	 * @return
	 */
	private double calculateLeft(Component c, FramingSpec f, Dimension parent, double right, STEdgeCache cache) {

		if (cache.containsLeft(c)) {
			return cache.getLeft(c);
		}

		if (!FramingSpec.isSupported(f.getLeftEdgeAttachment()) || f.getLeftEdgeAttachment() == FramingSpec.XmATTACHNONE) {
			return this.getPreferredWidth(c);
		}
		if (f.getLeftEdgeAttachment() == FramingSpec.XmATTACHFORM) {
			return f.getLeftEdge();
		}
		if (f.getLeftEdgeAttachment() == FramingSpec.XmATTACHOPPOSITEFORM) {
			return parent.getWidth() - f.getLeftEdge();
		}
		if (f.getLeftEdgeAttachment() == FramingSpec.XmATTACHSELFOPPOSITE) {
			return right - f.getLeftEdge();
		}
		if (f.getLeftEdgeAttachment() == FramingSpec.XmATTACHWIDGET) {
			if (f.getLeftTarget() == null) {
				return f.getLeftEdge();
			} else {
				FramingSpec f2 = this.getFramingSpec(f.getLeftTarget());
				STBounds bounds = this.calculateBoundsLeftRight(f.getLeftTarget(), f2, this.getSizeOfParentOfAttachedWidget(f.getLeftTarget()), cache);
				return bounds.getLocation() + bounds.getLength() + f.getLeftEdge();
			}
		}
		if (f.getLeftEdgeAttachment() == FramingSpec.XmATTACHOPPOSITEWIDGET) {
			if (f.getLeftTarget() == null) {
				return parent.getWidth() + f.getLeftEdge();
			} else {
				FramingSpec f2 = this.getFramingSpec(f.getLeftTarget());
				STBounds bounds = this.calculateBoundsLeftRight(f.getLeftTarget(), f2, this.getSizeOfParentOfAttachedWidget(f.getLeftTarget()), cache);
				return bounds.getLocation() + f.getLeftEdge();
			}
		}
		if (f.getLeftEdgeAttachment() == FramingSpec.XmATTACHPOSITION) {
			return parent.getWidth() / 100.0 * f.getLeftEdge();
		}
		throw new RuntimeException("unsupported left attachment: " + f.getLeftEdgeAttachment());
	}

	/**
	 * @param f
	 * @param parent
	 * @param left
	 * @return
	 */
	private double calculateRight(Component c, FramingSpec f, Dimension parent, double left, STEdgeCache cache) {

		if (cache.containsRight(c)) {
			return cache.getRight(c);
		}

		if (!FramingSpec.isSupported(f.getRightEdgeAttachment()) || f.getRightEdgeAttachment() == FramingSpec.XmATTACHNONE) {
			return left + this.getPreferredWidth(c);
		}
		if (f.getRightEdgeAttachment() == FramingSpec.XmATTACHFORM) {
			return parent.getWidth() - f.getRightEdge();
		}
		if (f.getRightEdgeAttachment() == FramingSpec.XmATTACHOPPOSITEFORM) {
			return f.getRightEdge();
		}
		if (f.getRightEdgeAttachment() == FramingSpec.XmATTACHSELFOPPOSITE) {
			return left + f.getRightEdge();
		}
		if (f.getRightEdgeAttachment() == FramingSpec.XmATTACHWIDGET) {
			if (f.getRightTarget() == null) {
				return parent.getWidth() - f.getRightEdge();
			} else {
				FramingSpec f2 = this.getFramingSpec(f.getRightTarget());
				STBounds bounds = this.calculateBoundsLeftRight(f.getRightTarget(), f2, this.getSizeOfParentOfAttachedWidget(f.getRightTarget()), cache);
				return bounds.getLocation() - f.getRightEdge();
			}
		}
		if (f.getRightEdgeAttachment() == FramingSpec.XmATTACHOPPOSITEWIDGET) {
			if (f.getRightTarget() == null) {
				return parent.getWidth() + f.getRightEdge();
			} else {
				FramingSpec f2 = this.getFramingSpec(f.getRightTarget());
				STBounds bounds = this.calculateBoundsLeftRight(f.getRightTarget(), f2, this.getSizeOfParentOfAttachedWidget(f.getRightTarget()), cache);
				return bounds.getLocation() + bounds.getLength() - f.getRightEdge();
			}
		}
		if (f.getRightEdgeAttachment() == FramingSpec.XmATTACHPOSITION) {
			return parent.getWidth() / 100.0 * f.getRightEdge();
		}
		throw new RuntimeException("unsupported right attachment: " + f.getRightEdgeAttachment());
	}

	/**
	 * @param f
	 * @param parent
	 * @param bottom
	 * @return
	 */
	private double calculateTop(Component c, FramingSpec f, Dimension parent, double bottom, STEdgeCache cache) {
		if (cache.containsTop(c)) {
			return cache.getTop(c);
		}

		if (!FramingSpec.isSupported(f.getTopEdgeAttachment()) || f.getTopEdgeAttachment() == FramingSpec.XmATTACHNONE) {
			return this.getPreferredHeight(c);
		}
		if (f.getTopEdgeAttachment() == FramingSpec.XmATTACHFORM) {
			return f.getTopEdge();
		}
		if (f.getTopEdgeAttachment() == FramingSpec.XmATTACHOPPOSITEFORM) {
			return parent.getHeight() - f.getTopEdge();
		}
		if (f.getTopEdgeAttachment() == FramingSpec.XmATTACHSELFOPPOSITE) {
			return bottom - f.getTopEdge();
		}
		if (f.getTopEdgeAttachment() == FramingSpec.XmATTACHWIDGET) {
			if (f.getTopTarget() == null) {
				return f.getTopEdge();
			} else {
				FramingSpec f2 = this.getFramingSpec(f.getTopTarget());
				STBounds bounds = this.calculateBoundsTopBottom(f.getTopTarget(), f2, this.getSizeOfParentOfAttachedWidget(f.getTopTarget()), cache);
				return bounds.getLocation() + bounds.getLength() + f.getTopEdge();
			}
		}
		if (f.getTopEdgeAttachment() == FramingSpec.XmATTACHOPPOSITEWIDGET) {
			if (f.getTopTarget() == null) {
				return parent.getHeight() + f.getTopEdge();
			} else {
				FramingSpec f2 = this.getFramingSpec(f.getTopTarget());
				STBounds bounds = this.calculateBoundsTopBottom(f.getTopTarget(), f2, this.getSizeOfParentOfAttachedWidget(f.getTopTarget()), cache);
				return bounds.getLocation() + f.getTopEdge();
			}
		}
		if (f.getTopEdgeAttachment() == FramingSpec.XmATTACHPOSITION) {
			return parent.getHeight() / 100.0 * f.getTopEdge();
		}
		throw new RuntimeException("unsupported top attachment: " + f.getTopEdgeAttachment());
	}

	/**
	 * @param c
	 * @return
	 */
	public FramingSpec getFramingSpec(Component c) {
		if (c.getParent() instanceof JViewport && c.getParent().getParent() instanceof JScrollPane) {
			c = c.getParent().getParent();
		}
		return this.components.get(c);
	}

	/**
	 * @see java.awt.LayoutManager2#getLayoutAlignmentX(java.awt.Container)
	 */
	public float getLayoutAlignmentX(Container target) {
		return 0.5f;
	}

	/**
	 * @see java.awt.LayoutManager2#getLayoutAlignmentY(java.awt.Container)
	 */
	public float getLayoutAlignmentY(Container target) {
		return 0.5f;
	}

	/**
	 * @param c
	 * @return
	 */
	private int getPreferredHeight(Component c) {
		return c.getPreferredSize().height;
	}

	/**
	 * @param c
	 * @return
	 */
	private int getPreferredWidth(Component c) {
		return c.getPreferredSize().width;
	}

	/**
	 * @param c
	 * @return
	 */
	private Dimension getSizeOfParentOfAttachedWidget(Component c) {
		if (c.getParent() instanceof JViewport && c.getParent().getParent() instanceof JScrollPane) {
			c = c.getParent().getParent();
		}
		Dimension size = c.getParent().getSize();
		Insets ins = c.getParent().getInsets();
		size.height -= (ins.top + ins.bottom);
		size.width -= (ins.left + ins.right);
		return size;
	}

	/**
	 * @see java.awt.LayoutManager2#invalidateLayout(java.awt.Container)
	 */
	public void invalidateLayout(Container target) {
		//
	}

	/**
	 * @see java.awt.LayoutManager#layoutContainer(java.awt.Container)
	 */
	public void layoutContainer(Container parent) {
		if (parent.getParent() instanceof JTabbedPane && ((JTabbedPane) parent.getParent()).getSelectedComponent() != parent) {
			parent.setBounds(0, 0, 0, 0);
			return;
		}
		STEdgeCache cache = new STEdgeCache();
		for (Iterator iter = this.components.keySet().iterator(); iter.hasNext();) {
			Component c = (Component) iter.next();
			if (!c.isVisible()) {
				c.setBounds(0, 0, 0, 0);
				continue;
			}
			FramingSpec f = (FramingSpec) this.components.get(c);
			Rectangle bounds = null;
			if (f == null) {
				bounds = new Rectangle(c.getLocation(), c.getPreferredSize());
			} else {
				Rectangle parBounds = new Rectangle(parent.getBounds());
				Insets ins = parent.getInsets();
				parBounds.width -= (ins.left + ins.right);
				parBounds.height -= (ins.top + ins.bottom);
				bounds = this.calculateBounds(c, f, parBounds.getSize(), cache);
				bounds.x += ins.left;
				bounds.y += ins.top;
			}
			c.setBounds(bounds);
		}
	}

	/**
	 * @see java.awt.LayoutManager2#maximumLayoutSize(java.awt.Container)
	 */
	public Dimension maximumLayoutSize(Container target) {
		return preferredLayoutSize(target);
	}

	/**
	 * @see java.awt.LayoutManager#minimumLayoutSize(java.awt.Container)
	 */
	public Dimension minimumLayoutSize(Container parent) {
		return preferredLayoutSize(parent);
	}

	/**
	 * @see java.awt.LayoutManager#preferredLayoutSize(java.awt.Container)
	 */
	public Dimension preferredLayoutSize(Container parent) {
		Dimension size = parent.getSize();
		for (int i = 0; i < parent.getComponentCount(); i++) {
			Rectangle bounds = parent.getComponent(i).getBounds();
			size.width = Math.max(size.width, bounds.x + bounds.width);
			size.height = Math.max(size.height, bounds.y + bounds.height);
		}
		return size;
	}

	/**
	 * @see java.awt.LayoutManager#removeLayoutComponent(java.awt.Component)
	 */
	public void removeLayoutComponent(Component comp) {
		this.components.remove(comp);
	}

	/**
	 * @param c
	 * @return
	 */
	public void setFramingSpec(Component c, FramingSpec f) {
		if (this.components.containsKey(c)) {
			this.components.put(c, f);
			update(c.getParent());
		}
	}

	synchronized private void update(final Container aContainer) {
		if (!update && aContainer != null) {
			update = true;
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					synchronized (STLayout.this) {
						STLayout.this.layoutContainer(aContainer);
						update = false;
					}
				};
			});
		}
	}

}
