package scm.sva.framework.vag.base;

import java.awt.Component;


/**
 * @author tej
 */
public class FramingSpec {

	public static final int XmATTACHFORM = 1;

	public static final int XmATTACHNONE = 2;

	public static final int XmATTACHOPPOSITEFORM = 3;

	public static final int XmATTACHOPPOSITEWIDGET = 4;

	public static final int XmATTACHPOSITION = 5;

	public static final int XmATTACHSELFOPPOSITE = 6;

	public static final int XmATTACHWIDGET = 7;

	/**
	 * @param attachment
	 * @return
	 */
	public static boolean isSupported(int attachment) {
		return attachment >= 1 && attachment <= 7;
	}

	/**
	 * @param i
	 * @return
	 */
	public static String toString(int i) {
		String attach = new String();
		if (i == 1) {
			attach = "XmATTACHFORM";
		} else if (i == 2) {
			attach = "XmATTACHNONE";
		} else if (i == 3) {
			attach = "XmATTACHOPPOSITEFORM";
		} else if (i == 4) {
			attach = "XmATTACHOPPOSITEWIDGET";
		} else if (i == 5) {
			attach = "XmATTACHPOSITION";
		} else if (i == 6) {
			attach = "XmATTACHSELFOPPOSITE";
		} else if (i == 7) {
			attach = "XmATTACHWIDGET";
		}
		return attach;
	}

	public int bottomEdge;

	public int bottomEdgeAttachment;

	public Component bottomTarget;

	public int leftEdge;

	public int leftEdgeAttachment;

	public Component leftTarget;

	public int overrideHeight = -1;

	public int overrideWidth = -1;

	public int overrideX = -1;

	public int overrideY = -1;

	public int rightEdge;

	public int rightEdgeAttachment;

	public Component rightTarget;

	public int topEdge;

	public int topEdgeAttachment;

	public Component topTarget;

	/**
	 * 
	 */
	public FramingSpec() {
		super();
	}

	/**
	 * @param leftAttach
	 * @param left
	 * @param leftTarget
	 * @param rightAttach
	 * @param right
	 * @param rightTarget
	 * @param topAttach
	 * @param top
	 * @param topTarget
	 * @param bottomAttach
	 * @param bottom
	 * @param bottomTarget
	 */
	public FramingSpec(int leftAttach, int left, Component leftTarget, int rightAttach, int right, Component rightTarget, int topAttach, int top, Component topTarget, int bottomAttach, int bottom, Component bottomTarget) {
		super();
		this.leftEdge = left;
		this.leftEdgeAttachment = leftAttach;
		this.leftTarget = leftTarget;
		this.rightEdge = right;
		this.rightEdgeAttachment = rightAttach;
		this.rightTarget = rightTarget;
		this.topEdge = top;
		this.topEdgeAttachment = topAttach;
		this.topTarget = topTarget;
		this.bottomEdge = bottom;
		this.bottomEdgeAttachment = bottomAttach;
		this.bottomTarget = bottomTarget;

		// combinations of none/self-opposite attachments not allowed
		if ((leftAttach == XmATTACHSELFOPPOSITE && rightAttach == XmATTACHSELFOPPOSITE) || (leftAttach == XmATTACHNONE && rightAttach == XmATTACHNONE) || (leftAttach == XmATTACHSELFOPPOSITE && rightAttach == XmATTACHNONE) || (leftAttach == XmATTACHNONE && rightAttach == XmATTACHSELFOPPOSITE)) {
			throw new IllegalArgumentException("Invalid int combination: left=" + toString(leftAttach) + ", right=" + toString(rightAttach));
		}
		if ((topAttach == XmATTACHSELFOPPOSITE && bottomAttach == XmATTACHSELFOPPOSITE) || (topAttach == XmATTACHNONE && bottomAttach == XmATTACHNONE) || (topAttach == XmATTACHSELFOPPOSITE && bottomAttach == XmATTACHNONE) || (topAttach == XmATTACHNONE && bottomAttach == XmATTACHSELFOPPOSITE)) {
			throw new IllegalArgumentException("Invalid int combination: top=" + toString(topAttach) + ", bottom=" + toString(bottomAttach));
		}
	}

	/**
	 * @param left
	 * @param right
	 * @param top
	 * @param bottom
	 */
	public FramingSpec(int left, int right, int top, int bottom) {
		this(XmATTACHFORM, left, XmATTACHFORM, right, XmATTACHSELFOPPOSITE, top, XmATTACHSELFOPPOSITE, bottom);
	}

	/**
	 * @param leftAttach
	 * @param left
	 * @param rightAttach
	 * @param right
	 * @param topAttach
	 * @param top
	 * @param bottomAttach
	 * @param bottom
	 */
	public FramingSpec(int leftAttach, int left, int rightAttach, int right, int topAttach, int top, int bottomAttach, int bottom) {
		this(leftAttach, left, null, rightAttach, right, null, topAttach, top, null, bottomAttach, bottom, null);
	}

	/**
	 * 
	 */
	private void applyOverride() {
		if (this.canOverride(this.topEdgeAttachment)) {
			if (this.overrideHeight >= 0) {
				this.topEdge = this.overrideHeight;
				this.topEdgeAttachment = FramingSpec.XmATTACHSELFOPPOSITE;
				this.overrideHeight = -1;
			}
		} else if (this.canOverride(this.bottomEdgeAttachment)) {
			if (this.overrideHeight >= 0) {
				this.bottomEdge = this.overrideHeight;
				this.bottomEdgeAttachment = FramingSpec.XmATTACHSELFOPPOSITE;
				this.overrideHeight = -1;
			}
		}

		if (this.canOverride(this.rightEdgeAttachment)) {
			if (this.overrideWidth >= 0) {
				this.rightEdge = this.overrideWidth;
				this.rightEdgeAttachment = FramingSpec.XmATTACHSELFOPPOSITE;
				this.overrideWidth = -1;
			}
		} else if (this.canOverride(this.leftEdgeAttachment)) {
			if (this.overrideWidth >= 0) {
				this.leftEdge = this.overrideWidth;
				this.leftEdgeAttachment = FramingSpec.XmATTACHSELFOPPOSITE;
				this.overrideWidth = -1;
			}
		}

		boolean change = this.overrideHeight >= 0 || this.overrideWidth >= 0 || this.overrideX >= 0 || this.overrideY >= 0;
		if (change) {
			StringBuffer sb = new StringBuffer();
			sb.append("X=" + this.overrideX + ", Y=" + this.overrideY + ", W=" + this.overrideWidth + ", H=" + this.overrideHeight);
			sb.append('\n');
			sb.append("leftEdgeAttachment: " + toString(this.leftEdgeAttachment));
			sb.append('\n');
			sb.append("rightEdgeAttachment: " + toString(this.rightEdgeAttachment));
			sb.append('\n');
			sb.append("topEdgeAttachment: " + toString(this.topEdgeAttachment));
			sb.append('\n');
			sb.append("bottomEdgeAttachment: " + toString(this.bottomEdgeAttachment));
//			MessageDlg.showError("Override von Framingspec nicht untersttzt: " + sb);
		}

		this.overrideHeight = -1;
		this.overrideWidth = -1;
		this.overrideX = -1;
		this.overrideY = -1;
	}

	private boolean canOverride(int attachment) {
		return attachment == FramingSpec.XmATTACHSELFOPPOSITE || attachment == FramingSpec.XmATTACHPOSITION;
	}

	/**
	 * @return
	 */
	public int getBottomEdge() {
		return bottomEdge;
	}

	/**
	 * @return
	 */
	public int getBottomEdgeAttachment() {
		return bottomEdgeAttachment;
	}

	public Component getBottomTarget() {
		return bottomTarget;
	}

	/**
	 * @return
	 */
	public int getLeftEdge() {
		return leftEdge;
	}

	/**
	 * @return
	 */
	public int getLeftEdgeAttachment() {
		return leftEdgeAttachment;
	}

	public Component getLeftTarget() {
		return leftTarget;
	}

	public int getOverrideHeight() {
		return overrideHeight;
	}

	public int getOverrideWidth() {
		return overrideWidth;
	}

	public int getOverrideX() {
		return overrideX;
	}

	public int getOverrideY() {
		return overrideY;
	}

	/**
	 * @return
	 */
	public int getRightEdge() {
		return rightEdge;
	}

	/**
	 * @return
	 */
	public int getRightEdgeAttachment() {
		return rightEdgeAttachment;
	}

	public Component getRightTarget() {
		return rightTarget;
	}

	/**
	 * @return
	 */
	public int getTopEdge() {
		return topEdge;
	}

	/**
	 * @return
	 */
	public int getTopEdgeAttachment() {
		return topEdgeAttachment;
	}

	public Component getTopTarget() {
		return topTarget;
	}

	/**
	 * @param bottomEdge
	 */
	public void setBottomEdge(int bottomEdge) {
		this.bottomEdge = bottomEdge;
	}

	/**
	 * @param bottomEdgeAttachment
	 */
	public void setBottomEdgeAttachment(int bottomEdgeAttachment) {
		this.bottomEdgeAttachment = bottomEdgeAttachment;
	}

	public void setBottomTarget(Component bottomTarget) {
		this.bottomTarget = bottomTarget;
	}

	/**
	 * @param leftEdge
	 */
	public void setLeftEdge(int leftEdge) {
		this.leftEdge = leftEdge;
	}

	/**
	 * @param leftEdgeAttachment
	 */
	public void setLeftEdgeAttachment(int leftEdgeAttachment) {
		this.leftEdgeAttachment = leftEdgeAttachment;
	}

	public void setLeftTarget(Component leftTarget) {
		this.leftTarget = leftTarget;
	}

	public void setOverrideHeight(int overrideHeight) {
		this.overrideHeight = overrideHeight;
		this.applyOverride();
	}

	public void setOverrideWidth(int overrideWidth) {
		this.overrideWidth = overrideWidth;
		this.applyOverride();
	}

	public void setOverrideX(int overrideX) {
		this.overrideX = overrideX;
		this.applyOverride();
	}

	public void setOverrideY(int overrideY) {
		this.overrideY = overrideY;
		this.applyOverride();
	}

	/**
	 * @param rightEdge
	 */
	public void setRightEdge(int rightEdge) {
		this.rightEdge = rightEdge;
	}

	/**
	 * @param rightEdgeAttachment
	 */
	public void setRightEdgeAttachment(int rightEdgeAttachment) {
		this.rightEdgeAttachment = rightEdgeAttachment;
	}

	public void setRightTarget(Component rightTarget) {
		this.rightTarget = rightTarget;
	}

	/**
	 * @param topEdge
	 */
	public void setTopEdge(int topEdge) {
		this.topEdge = topEdge;
	}

	/**
	 * @param topEdgeAttachment
	 */
	public void setTopEdgeAttachment(int topEdgeAttachment) {
		this.topEdgeAttachment = topEdgeAttachment;
	}

	/**
	 * @param topTarget
	 */
	public void setTopTarget(Component topTarget) {
		this.topTarget = topTarget;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("FramingSpec: ");

		sb.append("topEdge={");
		sb.append(this.topEdge);
		sb.append(", ");
		sb.append(toString(this.topEdgeAttachment));
		sb.append(", ");
		sb.append(this.topTarget == null ? "null" : this.topTarget.getName());
		sb.append("}, ");

		sb.append("bottomEdge={");
		sb.append(this.bottomEdge);
		sb.append(", ");
		sb.append(toString(this.bottomEdgeAttachment));
		sb.append(", ");
		sb.append(this.bottomTarget == null ? "null" : this.bottomTarget.getName());
		sb.append("}, ");

		sb.append("leftEdge={");
		sb.append(this.leftEdge);
		sb.append(", ");
		sb.append(toString(this.leftEdgeAttachment));
		sb.append(", ");
		sb.append(this.leftTarget == null ? "null" : this.leftTarget.getName());
		sb.append("}, ");

		sb.append("rightEdge={");
		sb.append(this.rightEdge);
		sb.append(", ");
		sb.append(toString(this.rightEdgeAttachment));
		sb.append(", ");
		sb.append(this.rightTarget == null ? "null" : this.rightTarget.getName());
		sb.append("}");

		return sb.toString();
	}

}
